import { processDateRanges } from './dateProcessor';
import React, { useState, useEffect, useCallback } from 'react';

const ElementCircles = () => {
    const [dateInput, setDateInput] = useState('');
    const [loading, setLoading] = useState(false);
    const [apiResponse, setApiResponse] = useState(null);
    const [spinning, setSpinning] = useState(true);
    const [circleStops, setCircleStops] = useState({ inner: 0, middle: 0, outer: 0 });
    const [soulmateResult, setSoulmateResult] = useState(null);

    const elements = [
        { name: 'Metal', gifUrl: '/api/placeholder/24/24', scale: 1 },
        { name: 'Water', gifUrl: '/api/placeholder/24/24', scale: 1.4 },
        { name: 'Wood', gifUrl: '/api/placeholder/24/24', scale: 1.5 },
        { name: 'Earth', gifUrl: '/api/placeholder/24/24', scale: 1.4 },
        { name: 'Fire', gifUrl: '/api/placeholder/24/24', scale: 1 }
    ];

    const getElementIndex = useCallback((elementName) => {
        if (!elementName) return 0;
        return elements.findIndex(e => e.name.toLowerCase() === elementName.toLowerCase());
    }, [elements]);

    const fetchData = async () => {
        setLoading(true);
        try {
            const deploymentId = 'AKfycbxB8_91lOvxYmNiBtLulA9BvEtB9P1S_FJlKKTWV7l3Pfr0lbv99HVjuAKzKK6xG6ve';
            const apiUrl = `https://script.google.com/macros/s/${deploymentId}/exec?date=${encodeURIComponent(dateInput)}`;
            const response = await fetch(apiUrl);
            const data = await response.json();
            setApiResponse(data);
            calculateStopPositions(data, "Subtle Destructive Betrayal Dates");
        } catch (error) {
            console.error('Error fetching data:', error);
        } finally {
            setLoading(false);
        }
    };

    const calculateStopPositions = (data, labelType) => {
        if (!data) return;

        const getElement = (planetData) => {
            const matchingDate = planetData.dates.find(d => d.label === labelType);
            return matchingDate ? matchingDate.element.replace(/[+-]/, '') : null;
        };

        const venusElement = getElement(data[0]);
        const earthElement = getElement(data[1]);
        const marsElement = getElement(data[2]);

        setSpinning(false);
        setCircleStops({
            inner: getElementIndex(venusElement) * 72,
            middle: getElementIndex(earthElement) * 72,
            outer: getElementIndex(marsElement) * 72
        });
    };

    const findSoulmate = () => {
        setSpinning(true);
        setSoulmateResult(null);
        
        setTimeout(() => {
            calculateStopPositions(apiResponse, "Aggressively Superempowering Dates");
            const dateRanges = processDateRanges(apiResponse);
            setSoulmateResult(dateRanges);
        }, 2000);
    };

    const createCircle = (radius, index) => {
        const tickMarks = [];
        const elementMarks = [];
        const rotationClass = spinning ? 
            ['rotate-outer', 'rotate-middle', 'rotate-inner'][index] : 
            'no-rotation';
        const counterRotationClass = spinning ?
            ['counter-rotate-outer', 'counter-rotate-middle', 'counter-rotate-inner'][index] :
            'no-rotation';
        
        // Create tick marks
        for (let i = 0; i < 24; i++) {
            const angle = (i * 15) * (Math.PI / 180);
            const x1 = radius * Math.cos(angle);
            const y1 = radius * Math.sin(angle);
            const x2 = (radius - 5) * Math.cos(angle);
            const y2 = (radius - 5) * Math.sin(angle);

            tickMarks.push(
                <line
                    key={`tick-${i}`}
                    x1={x1}
                    y1={y1}
                    x2={x2}
                    y2={y2}
                    stroke="white"
                    strokeWidth="1"
                    className="opacity-30"
                />
            );
        }

        // Create element marks
        elements.forEach((element, i) => {
            const angle = (i * 72) * (Math.PI / 180);
            const x = radius * Math.cos(angle);
            const y = radius * Math.sin(angle);

            elementMarks.push(
                <g key={`element-${i}`} transform={`translate(${x},${y})`}>
                    <circle
                        r={12}
                        fill="rgba(255, 255, 255, 0.1)"
                        stroke="white"
                        strokeWidth="1"
                        className="opacity-50"
                    />
                    <g mask="url(#circleMask)" className={counterRotationClass}>
                        <image
                            href={element.gifUrl}
                            width={24 * element.scale}
                            height={24 * element.scale}
                            x={-12 * element.scale}
                            y={-12 * element.scale}
                            preserveAspectRatio="xMidYMid meet"
                        />
                    </g>
                    <text
                        x="0"
                        y="22"
                        textAnchor="middle"
                        fontSize="8"
                        fill="white"
                        className={counterRotationClass}
                    >
                        {element.name}
                    </text>
                </g>
            );
        });

        const stopAngle = index === 0 ? circleStops.outer :
                         index === 1 ? circleStops.middle :
                         circleStops.inner;

        return (
            <g 
                key={`circle-${index}`} 
                className={`${rotationClass} circle-transition`}
                style={!spinning ? { transform: `rotate(${stopAngle}deg)` } : {}}
            >
                <circle
                    r={radius}
                    fill="none"
                    stroke="white"
                    strokeWidth="1"
                    className="opacity-50"
                />
                {tickMarks}
                {elementMarks}
            </g>
        );
    };

    return (
        <div className="container mx-auto px-4">
            <div className="background-container" />
            
            <div className="svg-container">
                <svg viewBox="-210 -210 420 420">
                    <defs>
                        <mask id="circleMask">
                            <circle cx="0" cy="0" r="12" fill="white"/>
                        </mask>
                    </defs>
                    <g>
                        {createCircle(186, 0)}
                        {createCircle(105, 1)}
                        {createCircle(30, 2)}
                    </g>
                </svg>
            </div>

            <div className="controls-container">
                <div className="flex flex-col md:flex-row gap-4 items-center justify-center">
                    <input
                        type="date"
                        value={dateInput}
                        onChange={(e) => setDateInput(e.target.value)}
                        className="px-4 py-2 rounded bg-gray-800 border border-gray-700 text-white"
                    />
                    <button
                        onClick={fetchData}
                        disabled={loading}
                        className="px-6 py-2 bg-blue-600 rounded hover:bg-blue-700 disabled:opacity-50 text-white"
                    >
                        {loading ? 'Loading...' : 'Submit'}
                    </button>
                    <button
                        onClick={findSoulmate}
                        disabled={!apiResponse}
                        className="px-6 py-2 bg-purple-600 rounded hover:bg-purple-700 disabled:opacity-50 text-white"
                    >
                        Find Soulmate
                    </button>
                </div>

                {soulmateResult && (
                    <div className="mt-8 text-center">
                        <h1 className="text-2xl text-white tracking-tight">
                            Your soulmate was born on {soulmateResult.ranges.join(', ')}
                            {!soulmateResult.complete && (
                                <span className="text-sm text-gray-400 block mt-2">
                                    Note: Jupiter yang or yin not found
                                </span>
                            )}
                        </h1>
                    </div>
                )}
            </div>
        </div>
    );
};

export default ElementCircles;